/*
 * Copyright (C) 2015 Canonical, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "simcardutils.h"

#include <QLocale>
#include <QDebug>

#include <phonenumbers/phonenumberutil.h>


SIMCardUtils::SIMCardUtils(QObject *parent)
    : QObject(parent),
      m_ofonoModem(new QDBusInterface("org.ofono", "/ril_0", "org.ofono.NetworkRegistration"))
{
    m_ofonoModem->installEventFilter(this);
    m_operatorName = m_ofonoModem->property("Name").toString();
    m_status = m_ofonoModem->property("Status").toString();
}

QString SIMCardUtils::operatorName() const
{
    return m_operatorName;
}

QString SIMCardUtils::formatPhoneNumberToCall(const QString &phoneNumber,
                                              const QString &defaultRegion,
                                              const QString &carrierCode,
                                              const QString &mySatePrefix,
                                              bool useOperatorForNonLocalNumbers,
                                              bool removePrefixFromLocalNumbers)
{
    std::string formattedNumber;
    i18n::phonenumbers::PhoneNumberUtil::PhoneNumberFormat pNFormat;

    // skip if it is a special number or a command
    if (phoneNumber.startsWith("#") || phoneNumber.startsWith("*")) {
        return phoneNumber;
    } else if (phoneNumber.startsWith("+")) {
        pNFormat = i18n::phonenumbers::PhoneNumberUtil::INTERNATIONAL;
    } else {
        pNFormat = i18n::phonenumbers::PhoneNumberUtil::NATIONAL;
    }

    i18n::phonenumbers::PhoneNumberUtil *phonenumberUtil = i18n::phonenumbers::PhoneNumberUtil::GetInstance();

    i18n::phonenumbers::PhoneNumber number;
    i18n::phonenumbers::PhoneNumberUtil::ErrorType error;
    QString region = !defaultRegion.isEmpty() ? defaultRegion : QLocale::system().name().split("_").last();
    error = phonenumberUtil->Parse(phoneNumber.toStdString(),
                                   region.toStdString(),
                                   &number);

    switch(error) {
    case i18n::phonenumbers::PhoneNumberUtil::INVALID_COUNTRY_CODE_ERROR:
        qWarning() << "Invalid country code for:" << phoneNumber;
        return "";
    case i18n::phonenumbers::PhoneNumberUtil::NOT_A_NUMBER:
        qWarning() << "The phone number is not a valid number:" << phoneNumber;
        return "";
    case i18n::phonenumbers::PhoneNumberUtil::TOO_SHORT_AFTER_IDD:
    case i18n::phonenumbers::PhoneNumberUtil::TOO_SHORT_NSN:
    case i18n::phonenumbers::PhoneNumberUtil::TOO_LONG_NSN:
        qWarning() << "Invalid phone number" << phoneNumber;
        return "";
    default:
        break;
    }

    bool internationalNumber = pNFormat == i18n::phonenumbers::PhoneNumberUtil::INTERNATIONAL;;

    std::string significantNumber;
    phonenumberUtil->GetNationalSignificantNumber(number,
                                                  &significantNumber);

    // check if number has destination Code
    int nationalDestinationLength = phonenumberUtil->GetLengthOfNationalDestinationCode(number);
    bool userCarrier = useOperatorForNonLocalNumbers && (nationalDestinationLength > 0);
    if (!internationalNumber && (nationalDestinationLength > 0)) {
        std::string nationalDestinationCode;
        std::string subscriberNumber;

        nationalDestinationCode = significantNumber.substr(0, nationalDestinationLength);
        subscriberNumber = significantNumber.substr(nationalDestinationLength, std::string::npos);

        // check if the destionation code is the same code used by the SIM
        if (!isRoaming() && (nationalDestinationCode == mySatePrefix.toStdString())) {
            userCarrier = false;
            phonenumberUtil->Parse(subscriberNumber,
                                   region.toStdString(),
                                   &number);

        }
    } else if (!internationalNumber && isRoaming()) {
        // if is Roamming and the number does not have prefix add the default one
        google::protobuf::uint64 nationalNumber = number.national_number();
        QString nationalNumberStr = QString("%1%2").arg(mySatePrefix).arg(nationalNumber);

        userCarrier = true;
        phonenumberUtil->Parse(nationalNumberStr.toStdString(),
                               region.toStdString(),
                               &number);
    }

    if (!internationalNumber && userCarrier) {
        phonenumberUtil->FormatNationalNumberWithCarrierCode(number,
                                                             carrierCode.toStdString(),
                                                             &formattedNumber);
    } else {
        phonenumberUtil->Format(number, pNFormat, &formattedNumber);
    }

    return QString::fromStdString(formattedNumber);
}

bool SIMCardUtils::eventFilter(QObject *obj, QEvent *ev)
{
    if ((obj == m_ofonoModem.data()) && (ev->type() == QEvent::DynamicPropertyChange)) {
        QDynamicPropertyChangeEvent *dev = static_cast<QDynamicPropertyChangeEvent*>(ev);
        if (dev->propertyName() == "Name") {
            m_operatorName = m_ofonoModem->property("Name").toString();
            Q_EMIT operatorNameChanged();
        } else if (dev->propertyName() == "Status") {
            m_status = m_ofonoModem->property("Status").toString();
        }
    }
    return QObject::eventFilter(obj, ev);
}


bool SIMCardUtils::isRoaming() const
{
    return m_status == "roaming";
}
